/*
 * Copyright 2011-2012 Janez Troha <janez.troha@gmail.com>
 * Copyright 2011-2012 Jiří Janoušek <janousek.jiri@gmail.com>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met: 
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this
 *    list of conditions and the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/* Anonymous function is used not to pollute environment */
(function(Nuvola){
	
	var get_play_button = function(){
		var play_button = $j('#player_play_button');
		if(play_button.length == 0) play_button = $j('#play_page span');
		if(play_button.length == 0) play_button = null;
		return play_button;
	}
	
	/**
	 * Creates 8tracks integration binded to Nuvola JS API
	 * @param Nuvola Nuvola JS API
	 */
	var Integration = function(){
		/* Overwrite default commnad function */
		Nuvola.command = Nuvola.bind(this, this.command);
		
		/* For debug output */
		this.name = "8tracks";
		
		/* Let's run */
		this.state = Nuvola.STATE_NONE;
		this.update();
		// Periodically update state
		this.timeout = setInterval(Nuvola.bind(this, this.update), 500);
	};

	/**
	 * Updates current playback state
	 */
	Integration.prototype.update = function(){
		
		var album = null;
		var album_art = null;
		var artist = null;
		var song = null;
		var state = Nuvola.STATE_NONE;
		var can_prev = false;
		var can_next = false;
		var can_thumbs_up = false;
		var can_thumbs_down = false;
		var can_favorite = $("#profile_link").length != 0;
		
		try{
			song = $(".track.now_playing .t").text();
			album = $(".track.now_playing .album .detail").text();
			artist = $(".track.now_playing .a").text();
			album_art = $("#play_area .cover")[0].src;
			can_next = true; // FIXME
			
		}
		catch(e){
			//alert("Unable to grab info:" + e.message);
		}
		try{	
			var play_button = $j('#player_play_button');
			play_button = play_button.length == 0 
				? null : play_button[0];
			
			//~ if(! play_button){
				//~ play_button = $j('#play_page');
			//~ play_button = play_button.length == 0 
				//~ ? null : play_button[0];
			//~ }
			
			var pause_button = $j('#player_pause_button');
			pause_button = pause_button.length == 0 
				? null : pause_button[0];
			
			if(play_button && play_button.style.display != "none"){
				state = Nuvola.STATE_PAUSED;
			}
			else if(pause_button && pause_button.style.display != "none"){
				state = Nuvola.STATE_PLAYING;
			}
		}
		catch(e){
			alert("Unable to grab state: " + e.message);
		}
		
		this.state = state;
		Nuvola.dataChanged(
			song, artist, album, album_art,
			state, can_prev, can_next,
			can_thumbs_up, can_thumbs_down, can_favorite);
	}
	
	
	/**
	 * Command handler
	 * @param cmd command to execute
	 */
	Integration.prototype.command = function(cmd){
		try{
			console.log(this.name + ": " + cmd + ", " + this.state);
			switch(cmd){
			case Nuvola.CMD_PAUSE:
				if(this.state == Nuvola.STATE_PLAYING)
					window.$j('#player_play_button').trigger('click');
				break;
			case Nuvola.CMD_PLAY:
				if(this.state != Nuvola.STATE_PLAYING)
					window.$j('#player_play_button').trigger('click');
				break;
			case Nuvola.CMD_TOGGLE:
				get_play_button().trigger('click');
				break;
			case Nuvola.CMD_NEXT_SONG:
				window.$j('#player_skip_button').trigger('click');
				break;
			case Nuvola.CMD_FAVORITE:
				window.$j('#like_button input[type="submit"]').trigger("click");
				break;
			default:
				// Other commands are not supported
				throw {"message": "Not supported."};
			}
			console.log(this.name + ": comand '" + cmd + "' executed.");
		}
		catch(e){
			// API expects exception to be a string
			throw (this.name + ": " + e.message);
		}
	}
	
	
 Nuvola.integration = new Integration(); // Singleton ;-)
})(window._Nuvola);

