/*
 * Copyright 2011-2012 Jiří Janoušek <janousek.jiri@gmail.com>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met: 
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this
 *    list of conditions and the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/* Anonymous function is used not to pollute environment */
(function(Nuvola){
	var ACTIVITY_INTERVAL = 15*60*1000;
	/**
	 * Creates Google Music integration binded to Nuvola JS API
	 */
	var Integration = function(){
		/* Overwrite default commnad function */
		Nuvola.onMessageReceived = Nuvola.bind(this, this.messageHandler);
		
		/* For debug output */
		this.name = "Google Music";		
		
		/* Let's run */
		this.state = Nuvola.STATE_NONE;
		this.can_thumbs_up = null;
		this.can_thumbs_down = null;
		this.can_prev = null;
		this.can_next = null;
		this.update();
		this.extraFeatures();
	};
	
	/**
	 * Updates current playback state
	 */
	Integration.prototype.update = function(){
		// Default state
		var state = Nuvola.STATE_NONE;
		var album = null;
		var album_art = null;
		var artist = null;
		var song = null;
		var can_prev;
		var can_next;
		var can_thumbs_up;
		var can_thumbs_down;
			
		try{
			var album_art = document.getElementById('playingAlbumArt').src;
			var artist = document.getElementById('playerArtist').firstChild.title;
			var song = document.getElementById('playerSongTitle').firstChild.title;
		}
		catch(e){
			//~ console.debug("Unable to obtain song info: " + e.message);
		}
		
		try{
			var play_pause =  document.getElementById('playPause');
			var pp_pressed = play_pause.getAttribute("aria-pressed");
			var pp_disabled = play_pause.getAttribute("aria-disabled");
			var state;
			if(pp_pressed === null && pp_disabled == null){
				state = Nuvola.STATE_NONE;
			}
			else if(pp_disabled == "true"){
				state = Nuvola.STATE_NONE;
			}
			else if(pp_pressed === null && pp_disabled == "false"){
				state = Nuvola.STATE_PAUSED;
			}
			else{
				state = ( pp_pressed == "true") ? Nuvola.STATE_PLAYING : Nuvola.STATE_PAUSED;
			}
			
			if(state != Nuvola.STATE_NONE){
				var disabled = document.getElementById('rew').getAttribute("aria-disabled")
				can_prev = disabled !== null && disabled !== "true";
				disabled = document.getElementById('ff').getAttribute("aria-disabled")
				can_next = disabled !== null && disabled !== "true";
			}
			else{
				can_prev = can_next = false;
			}
			
		}
		catch(e){
			can_prev = can_next = false;
		}
		
		try{
			can_thumbs_up = document.getElementById('thumbsUpPlayer').style.display != "none";
		}
		catch(e){
			can_thumbs_up = false;
		}
		
		try{
			can_thumbs_down = document.getElementById('thumbsDownPlayer').style.display != "none";
		}
		catch(e){
			can_thumbs_down = false;
		}
		
		this.state = state;
		
		// Submit data to Nuvola backend
		Nuvola.updateSong(song, artist, album, album_art, state);
		
		if(this.can_prev !== can_prev){
			this.can_prev = can_prev;
			Nuvola.updateAction(Nuvola.ACTION_PREV_SONG, can_prev);
		}
		if(this.can_next !== can_next){
			this.can_next = can_next;
			Nuvola.updateAction(Nuvola.ACTION_NEXT_SONG, can_next);
		}
		if(this.can_thumbs_up !== can_thumbs_up){
			this.can_thumbs_up = can_thumbs_up;
			Nuvola.updateAction(Nuvola.ACTION_THUMBS_UP, can_thumbs_up);
		}
		if(this.can_thumbs_down !== can_thumbs_down){
			this.can_thumbs_down = can_thumbs_down;
			Nuvola.updateAction(Nuvola.ACTION_THUMBS_DOWN, can_thumbs_down);
		}
		
		setTimeout(Nuvola.bind(this, this.update), 500);
	}
	
	/**
	 * Command handler
	 * @param cmd command to execute
	 */
	Integration.prototype.messageHandler = function(cmd){
		/* SJBpost is an internal Google Music JavaScript function */
		try{
			switch(cmd){
			case Nuvola.ACTION_PLAY:
				if(this.state != Nuvola.STATE_PLAYING) SJBpost("playPause");
				break;
			case Nuvola.ACTION_PAUSE:
				SJBpost("pauseSong");
				break;
			case Nuvola.ACTION_TOGGLE_PLAY:
				SJBpost("playPause");
				break;
			case Nuvola.ACTION_PREV_SONG:
				SJBpost("prevSong");
				break;
			case Nuvola.ACTION_NEXT_SONG:
				SJBpost("nextSong");
				break;
			case Nuvola.ACTION_THUMBS_UP:
				// Emulate click on thumbs up button
				Nuvola.clickOnElement(document.getElementById('thumbsUpPlayer'));
				break;
			case Nuvola.ACTION_THUMBS_DOWN:
				// Emulate click on thumbs down button
				Nuvola.clickOnElement(document.getElementById('thumbsDownPlayer'));
				break;
			default:
				// Other commands are not supported
				throw {"message": "Not supported."};
			}
			console.log(this.name + ": comand '" + cmd + "' executed.");
		}
		catch(e){
			// API expects exception to be a string!
			throw (this.name + ": " + e.message);
		}
	}
	
	Integration.prototype.extraFeatures = function(){
		if(Nuvola.config.googlebar === false){
			var googlebar = document.getElementById("gb");
			if(googlebar){
				var googlebarParent = googlebar.parentNode;
				if(googlebarParent){
					/* Remove Google Bar */
					googlebarParent.removeChild(googlebar);
				}
			}
		}
		if(Nuvola.config.simulateActivity === true){
			setTimeout(Nuvola.bind(this, this.simulateActivity), ACTIVITY_INTERVAL);
		}
	}
	
	Integration.prototype.simulateActivity = function(){
		var elm = document.getElementById("insideBox");
		if(elm){
			console.log("Click on " + elm);
			Nuvola.clickOnElement(elm);
			setTimeout(Nuvola.bind(this, this.simulateActivity), ACTIVITY_INTERVAL);
		}
	}
	/* Store reference */ 
	Nuvola.integration = new Integration(); // Singleton

// Immediately call the anonymous function with Nuvola JS API main object as an argument.
// Note that "this" is set to the Nuvola JS API main object.
})(this);
