/*
	Nuvola Player :: Cloud music integration
	
	This script provides integration of the Rdio streaming service
	on the JavaScript side. It's executed when Rdio page is completely
	loaded.


	Copyright 2012 Stefan Lohmaier <stefan.lohmaier@googlemail.com>

This program is free software: you can redistribute it and/or modify it 
under the terms of the GNU General Public License version 3, as published 
by the Free Software Foundation.

This program is distributed in the hope that it will be useful, but 
WITHOUT ANY WARRANTY; without even the implied warranties of 
MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along 
with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/* Anonymous function is used not to pollute environment */
(function(Nuvola){
	/**
	 * Creates Rdio integration binded to Nuvola JS API
	 */
	var Integration = function(){
		/* Overwrite default commnad function */
		Nuvola.command = Nuvola.bind(this, this.command);
		
		/* For debug output */
		this.name = "Rdio";		
		
		/* Let's run */
		this.state = Nuvola.STATE_NONE;
		this.update();
		this.timeout = setInterval(Nuvola.bind(this, this.update), 500);
	};


	/**
	 * Updates current playback state
	 */
	Integration.prototype.update = function(){
		// Default state
		var state = Nuvola.STATE_NONE;
		var album = null;
		var album_art = null;
		var artist = null;
		var song = null;
		var can_prev;
		var can_next;
		var can_thumbs_up = false;
		var can_thumbs_down = false;
		var can_favorite = false;
			
		try{
			var playingTrack = R.Services.Player.model.get("playingTrack").attributes;				
			album_art = playingTrack.icon;
			album = playingTrack.album;
			artist = playingTrack.artist;
			song = playingTrack.name;
		}
		catch(e){
			//~ console.debug("Unable to obtain song info: " + e.message);
		}
		
		try{
			var state = Nuvola.STATE_NONE;

			var rdiostate = R.Services.Player.model.attributes.playState;//new

			switch (rdiostate){
				case 0: state = Nuvola.STATE_PAUSED; break;
				case 1: state = Nuvola.STATE_PLAYING; break;
			}
			
			if(state != Nuvola.STATE_NONE){
				can_prev = can_next = true;
			}
			else{
				can_prev = can_next = false;
			}
			
		}
		catch(e){
			can_prev = can_next = false;
		}

		this.state = state;
		
		// Submit data to Nuvola backend
		Nuvola.dataChanged(
			song, artist, album, album_art,
			state, can_prev, can_next,
			can_thumbs_up, can_thumbs_down, can_favorite);
	}
	
	/**
	 * Command handler
	 * @param cmd command to execute
	 */
	Integration.prototype.command = function(cmd){
		try{
			switch(cmd){
			case Nuvola.CMD_PLAY:
				if(this.state != Nuvola.STATE_PLAYING) R.Services.Player.playPause();
				break;
			case Nuvola.CMD_PAUSE:
				if(this.state == Nuvola.STATE_PLAYING) R.Services.Player.playPause();
				break;
			case Nuvola.CMD_TOGGLE:
				R.Services.Player.playPause();
				break;
			case Nuvola.CMD_PREV_SONG:
				R.Services.Player.previous();
				break;
			case Nuvola.CMD_NEXT_SONG:
				R.Services.Player.next();
				break;
			default:
				// Other commands are not supported
				throw {"message": "Not supported."};
			}
			console.log(this.name + ": comand '" + cmd + "' executed.");
		}
		catch(e){
			// API expects exception to be a string!
			throw (this.name + ": " + e.message);
		}
	}

	/* Store reference */ 
	Nuvola.integration = new Integration(); // Singleton
	
// Call anonymous function with Nuvola object as an argument
})(window._Nuvola);
